function [res, perInf, tta, tea]=simulateMyc_placed(str,simtime,minAge,maxAge,P,maxInfection,dt,keep,itime)
% simulate: Starts a simulation run.
%
% Function is called by all Example files including mycorrhization
%
% Parameters:
% str           initial L-System string
% simtime       simulation time
% minAge        minimal age where root is succeptible for infection
% maxAge        maximal age where root is succeptible for infection
% P             probability that one unit of root gets infected within 1 day
% maxInfection  maximal percentage of infected root length of a root system
% (dt)          time step iof primary/secondary infections and anastomosis
%               (default: dt=1)
% (keep)        keep intermediate results (default: keep=false)
% (itime)       initial time (default: itime=0)
%
% res           resulting L-System string at the final time or list of
%               L-System strings (several output times)
% perInf        percent of infected segments
% tta           tip-tip anatomosis coordinates for visualization
% tea           tip-edge anatomosis coordintate for visualization
%
% See also: ExampleMyc, applyMycRules
%
% Copyright 2008-2014 Daniel Leitner & Andrea Schnepf. See license.txt for details.
%

global parameters

IID = 1;

if nargin<7
    dt = 1;
end

if nargin<8
    keep = 0;
end

if nargin<9
    itime=0;
end

nof = ceil(simtime/dt);
if keep
    res = cell(1,nof);
    perInf = zeros(1,nof);
    tta = cell(1,nof);
    tea = cell(1,nof);
end

s = RandStream.create('mlfg6331_64','Seed','shuffle'); % Myc simulation uses this stream

tta_ = []; % coordinates for tta vizualisation
tea_ = []; % coordinates for tta vizualisation

% default position
pos.x=[0;0;0];
pos.C=[0,0,0];
pos.a=0.1;
pos.t=0;
pos.R=eye(3);

tic
for i = 1 : nof
    
    ct = itime+dt*i; % current time
    
    fprintf('\nSimulation: day %g (dt=%g) string size %g \n',ct,dt,size(str,1));    
    str = applyMycRules(str,dt,s);
    
    
    %%
    % New secondary infection
    %   
    fprintf('Secondary infection \n');
    [x1,x2,~,colors,time,type,ind]=getSegments(str); % ind: index within the l-system string
    list=getSusceptibles(str,x1,x2,time,ind,ct,s); % list of susceptible segment indices
        
    counter=0;
    incl=0; % increase in infected leng
    
    % infect all susceptible segments by secondary infection
    if ~isempty(list)
        
        % number of root segmentes that are infected times length
        infectedLength = sum(((str(ind,1)==1 & str(ind,5)<100)&(str(ind,6)>0)).*str(ind,3));
        allLength = sum((str(ind,1)==1 & str(ind,5)<100).*str(ind,3));
        
        list = unique(list);
        
        % sort segment indices such that the L-system string indices are in
        % ascending order
        listStrInd = ind(list);
        [~,I] = sort(listStrInd);
        list=list(I);
        
        for i0 = 1:length(list)
            
            percentInfected = (infectedLength+incl)/allLength;
            k  = list(i0);
            j = ind(k)+counter*9;     % ind(k): index of segment k within the L-system string            
            
%             if percentInfected < maxInfection && str(j,6)==0 && ...
%                     age > minAge && age < maxAge
            if percentInfected < maxInfection && str(j,6)==0 
                
                %fprintf('Secondary infection: %g/%g, %g susceptible segments, letter index %g \n',i,nof,length(list), j);
                
                if str(j,1) == 1 % type == F
                    
                    % make segment infected and change creation time to
                    % infection time
                    str(j,6)=2;    % secondary infection
                    str(j,4)=ct;   % current time
                    
                    % set up mycelium
                    % provide initial information from current segment
                    myc = letter(403,[1,0,IID],[],pos);
                    IID=IID+1;
                    new = [letter('['); letter('r',[2*pi*rand(s),pi/2]); ...
                        myc; letter(']')];
                    
                    rootcolor = letter('C',colors(k,1:3));
                    blue = letter('C',[100 149 237]./250);
                    str=[str(1:j-1,:); blue; str(j,:);  ...
                        new; rootcolor; str(j+1:end,:)];
                    
                    ind(k) = ind(k)+counter*9 +1; % update ind for primary infection
                    
                    incl = incl + str(j,3);
                    counter=counter+1;
                    
                else
                    if (str(j,1)~=300) % 300 is fine, that could happen
                        disp(['Error in secondary infection, found type: ' num2str(str(j,1))]);
                    end
                end
            else 
                
                fprintf('already infected: %g',str(j,6) );
                
            end
            
        end % for
        
    end
    
    
    %%
    % new primary infection
    %      
    fprintf('Primary infection \n');
    if counter>0 % if we changed something draw again
        [x1,x2,~,colors,time,type,ind]=getSegments(str);
    end % TODO check if it works without that update
    
    counter=0;  % counter of infected root segments
    incl=0;
    
    if ~isempty(ind)
        
        infectedLength = sum(((str(ind,1)==1 & str(ind,5)<100)&(str(ind,6)>0)).*str(ind,3));
        allLength = sum((str(ind,1)==1 & str(ind,5)<100).*str(ind,3));
        
        for jj=1:size(ind,1) % for all segments
            
            percentInfected = (infectedLength+incl)/allLength;
            age = ct-time(jj);
            rOrder = type(jj);
            if rOrder<100
                rlt=parameters{rOrder}.rlt(1);
            else
                rlt=10;
            end
            maxRootAge = min(30,rlt);
            P_ =(1-age./maxRootAge).*P;  % probability is dependent on root age
            l=norm(x2(jj,:)-x1(jj,:));
            z=x2(jj,3)/7.5; 
            rr=(1+z)*4-z*3.15; 
            
            if ( type(jj)<100 && age > minAge && age < maxAge && ...
                    str(ind(jj),5)<100 && rand(s) <= (1 - (1-P_).^(dt*l)) && ...    % 
                    abs(x1(jj,3)) < 5 &&  abs(x2(jj,3))<5 &&...             % for placed inocculum
                    str(ind(jj),6)==0 &&  percentInfected < maxInfection  )
                
                %fprintf('Primary infection: %g/%g, letter index %g \n',i,nof,ind(jj));
                
                % make segment infected and change creation time to
                % infection time
                str(ind(jj)+counter*9,6)=1;   % primary infection
                str(ind(jj)+counter*9,4)=ct;
                incl = incl + str(ind(jj)+counter*9,3);
                
                % set up mycelium
                % provide initLabels information from current segment and
                % time step
                myc = letter(403,[1,0,IID],[],pos);
                IID=IID+1;
                new = [letter('['); letter('r',[2*pi*rand(s),pi/2]); ...
                    myc; letter(']')];
                
                rootcolor = letter('C',colors(jj,1:3));
                red = letter('C',[1 0 0]);
                
                str=[str(1:ind(jj)+counter*9-1,:); red; str(ind(jj)+counter*9,:);...
                    new; rootcolor; str(ind(jj)+counter*9+1:end,:)]; % todo: make faster
                
                counter=counter+1;
                                
            end
        end
        
    end
    
    str = updatePos(str);
    
    
    %%
    %---------------anastomosis----------------------------------------------------
    % tip - tip
    %
    
   [I,J] = findTTACanditates(str,dt,s);    
   fprintf('Tip-tip anastomosis (%g canditates) \n',size(I,1)); 
    
    for k = 1 : length(I)
        
        % delete letter
        if str(I(k),1)==400 && str(J(k),1)==400 % otherwise tip was already removed
            
            % for visualisation
            x1 = str(I(k)+1,1:3);
            x2 = str(J(k)+1,1:3);
            tta_ = [tta_; x1,x2];            
            
            % Tip I
            lr = str(I(k),2); % letter rows
            str(I(k):I(k)+lr-1,:) = [];
            % insert segment
            new = letter('|');
            str = [str(1:I(k)-1,:); new; str(I(k):end,:)];
            % modify remaining indices, TODO k+1:end would be enough
            I(I>I(k)) = I(I>I(k)) - lr + size(new,1);
            J(J>I(k)) = J(J>I(k)) - lr + size(new,1);
            
            % Tip J
            lr = str(J(k),2); % letter rows
            str(J(k):J(k)+lr-1,:) = [];
            % insert |
            new = letter('|');
            str = [str(1:J(k)-1,:); new; str(J(k):end,:)];
            % modify remaining indices, TODO k+1:end would be enough
            I(I>J(k)) = I(I>J(k)) - lr + size(new,1);
            J(J>J(k)) = J(J>J(k)) - lr + size(new,1);            
            
            %disp('connected tip - tip');
            
        else
            
            %disp('X tip-tip');
            
        end
        
    end
    
    
    %%
    % tip - edge anastomosis
    %   
    
    [I,J,mid] = findTEACanditates(str,dt,s);    
    fprintf('Tip-hypahe anastomosis (%g canditates) \n',size(I,1));
    
    for k = 1 : length(I)
        
        % Tip I
        % delete letter
        if str(I(k),1)==400 % otherwise tip was already removed
            
            % for visualization
            x1 = str(I(k)+1,1:3);
            x2 = mid(J(k),:);
            tea_ = [tea_; x1,x2];
            
            lr = str(I(k),2); % letter rows
            str(I(k):I(k)+lr-1,:) = [];
            % insert segment
            new = letter('|');
            str = [str(1:I(k)-1,:); new; str(I(k):end,:)];
            % modify remaining indices, TODO k+1:end would be enough
            I(I>I(k)) = I(I>I(k)) - lr + size(new,1);                    
            
            %disp('connected tip - edge');
            
        else
            
            %disp('X tip-edge');
            
        end
                
        
    end
    
    %%
    
    if keep
        res{i} = str;
        perInf(i) = percentInfected;
        tta{i} = tta_;
        tea{i} = tea_;
    end
    
end
toc

if (~keep)
    res{1} = str;
    perInf(1) = percentInfected;
    tta{1} = tta_;
    tea{1} = tea_;
end

