function [seg_info, tip_info]=convert2RSWMSinput(str, pos)
% convert2RSWMSinput: converts the l-system string to RSWMS input parameter
%
% Parameters:
% str           l-system string
% (pos)         initial position (default: pos.x=[0;0;0], pos.a=0.1, pos.R=
%               eye(3)) 
%
% seg_info      segments <Sx11 double> (#, X, Y, Z, connected, order,
%               branchnumber, length, surface, mass (=0), age) 
% tip_info      root tips <Tx8 double> (#, X, Y, Z, segment, order, branchid,
%               length, 0, 0, 0)  
%
% segments: 
% #                 are not sorted 
% connected         id of the previous segment
% order             is actually the root type
% branchnumber      is the tip id 
% age               actually the creation time
%
% #                 tip id sorted (index==id)
% segment           last segment of the branch
% order             is actually the root type
% branchid          branchid==tipid
% length            branch length
%
%
% See also: getPolylines, getGraph, getSegments.
%
% Copyright 2013 Daniel Leitner. See license.txt for details.
%

if (nargin<2)
    pos.x=[0;0;0];
    pos.a=0.1;
    pos.R=eye(3);
end

pos.prev = 0; % previous segment
pos.bn = 1; % branch number
pos.sC = 1; % segment counter

c = 1; % segment counter
stkPtr = 1; % stack pointer
bc = 0;

N = getTotal(str,'f');
X = zeros(N,3);
prev = zeros(N,1);
type = zeros(N,1);
branchnumber = zeros(N,1);
len = zeros(N,1);
surface = zeros(N,1);
time = zeros(N,1);

tips = zeros(N,1);

i=1;
while i<=size(str,1)
    
    l = str(i,:); % letter i
    
    if l(1)==300 % growing tip = F
        l(1) = 1;
        l(3) = 1e-9; % TODO, i dont know that length
    end
    
    switch l(1)
        case 4 % #
            pos.a = l(3);
        case 3 % C
        case 2 % f
            pos.x = pos.x + l(3)*pos.R(:,1);
        case 1 % F
            
            if l(3)>0 % if this segment has a length
                pos.x = pos.x + l(3)*pos.R(:,1);
                X(c,:) = pos.x;
                prev(c) = pos.prev;
                type(c) = l(5); 
                branchnumber(c) = pos.bn;
                len(c) = l(3);
                surface(c) = 2*pos.a*pi*l(3);
                time(c) = l(4);
                pos.prev=c;
                c=c+1;
                pos.sC=pos.sC+1;
            end
            
        case 7 % +
            a = l(3);
            pos.R = pos.R * [ cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1 ];
        case 8 % -
            a = l(3);
            pos.R = pos.R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case 9 % &
            a = l(3);
            pos.R = pos.R * [ cos(a) 0  -sin(a) ; 0 1 0; sin(a) 0 cos(a) ];
        case 10 % ^
            a = l(3);
            pos.R = pos.R * [ cos(a) 0 sin(-a) ; 0 1 0; -sin(a) 0 cos(a)];
        case 11 % \
            a = l(3);
            pos.R = pos.R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
        case 12 % /
            a = l(3);
            pos.R = pos.R * [ 1 0 0; 0 cos(a) sin(a) ; 0 -sin(a) cos(a) ];
        case 13 % |
            pos.R = pos.R * [ cos(pi) sin(pi) 0; -sin(pi) cos(pi) 0; 0 0 1];
        case 14 % r
            a = l(4); % beta
            pos.R = pos.R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
            a = l(3); % alpha
            pos.R = pos.R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case 5 % [ push the stack
                        
            stack(stkPtr) = pos;
            stkPtr = stkPtr +1;
            
            if branch(str(i:end,:))
                bc=bc+1; % increase branch counter
                pos.bn = bc; 
            end                        
            
            pos.sC = 1;
            
            
        case 6 % ] pop the stack
            
            if pos.sC>1
                tips(pos.bn) = pos.prev;  % save the segment belonging to the branch
            end
            
            stkPtr = stkPtr -1;
            pos = stack(stkPtr);
    end
    
    i = i + l(2); % jump to next letter
    
end

seg_info = [(1:c-1)',X(:,1),X(:,2),X(:,3),prev,type,branchnumber,len,surface,zeros(c-1,1),time ]; 

tips = tips(1:bc);
blen = zeros(bc,1);
for i = 1 : bc
    blen(i) = sum(len(branchnumber==i));
end        
tip_info = [(1:bc)', X(tips,1),X(tips,2),X(tips,3), tips, type(tips), (1:bc)', ...
    blen, zeros(bc,1), zeros(bc,1), zeros(bc,1) ];

% special needs of rswms %(#,X,Y,Z,connected,order,branchnumber,length,surface, mass (=0), age)
seg_info(:,11) = round(seg_info(:,11)); % round age
[~,ix] = sort(seg_info(:,7)); % sort by branchnumber
seg_info = seg_info(ix,:); 

seg0 = [1,0,0,0,0,0,0,0,0,0,0]; % artificial origin
seg_info(:,1)=seg_info(:,1)+1; % segment number
seg_info(:,5)=seg_info(:,5)+1; % prev number
seg_info = [seg0; seg_info];



function e = branch(str)
% checks if subbranch is a branch and not [[b1][b2]]
e=0;
stkPtr=1;
i=1;
while i<=size(str,1)
    l = str(i,:); % letter i
    if l(1)==300 % growing tip = F
        l(1) = 1;
        l(3) = 1e-9;
    end
    switch l(1)
        case 1 % F
            if l(3)>0 && stkPtr==2 % if this segment has a length
                e=1;
                break; 
            end
        case 5 % [ push the stack
            stkPtr = stkPtr+1;
        case 6 % ] pop the stack
            stkPtr = stkPtr-1;
            if stkPtr==1
                break;
            end
    end
    i = i + l(2); % jump to next letter
end


