function rootsystem = createMonocotRS(depth,firstB,delayB,maxB,dzB,firstCR,delayCR,nCR,delayRC,dzRC)
% createRootSystem: creates multiple taproots 
%
% Parameters:
% depthB             planting depth (cm)
% firstB             first basal roots [mean, sd] (days)
% delayB             time delay between basal roots [mean, sd] (days)
% maxB               maximal number of basal roots
% dzB                distance upwards the hypocotyl per basal root [mean,
%                    sd] (cm) 
% firstRC            first shootborne or root crown (days)
% nCR                number of root crowns 
% delayCR            delay between shootbore crown roots (days)
% delayRC            delay between root crowns (days)
% dzRC               distance between root crowns along shoot (cm)   
%
% rootsystem     initial l-system string of the root system
%
% See also: completeParameters, applyRules.
%
% Copyright 2015 Daniel Leitner & Andrea Schnepf. See license.txt for details.
%     

global parameters;

if isempty(parameters)
    error('createRootSystems: parameters not set use completeParameters to set global parameters');
end

if nargin<9
    delayRC = Inf; % there is only one root crown
end

if nargin<10
    dzRC = 0;
end
  
% add a zero to all inputs
firstB = [firstB,0];
delayB = [delayB,0];
dzB = [dzB,0];
firstCR = [firstCR,0];
delayCR = [delayCR,0];
nCR = [nCR,0];
delayRC = [delayRC,0];
dzRC = [dzRC,0];

%
% initialize
%
taproot = letter(303,1); % create tap root
basal = letter(303,4); % create basal root
shootborne = letter(303,5); % shootborne root
push = letter('[');
pop = letter(']');
pitchdown  = letter('^',pi/2);
turnaround = letter('|');

%
% set up tap and basal roots
%
N = round((365-firstB(1))/delayB(1));  % a guess how many basal roots are needed
N = min(N,maxB); % limit to maxB basal roots

ct = firstB(1)+randn()*firstB(2);
cz = 0;
str = [push; taproot; pop];
for i = 1 : N
    cz = cz+dzB(1)+randn()*dzB(2);
    fup = letter('f',cz);  % we could move up the mesocotyl
    ct = ct + delayB(1)+randn()*delayB(2);
    if ct>0
        root = letter(301,[0 ct],basal); % delay
    else
        root = basal;
    end
    str = [str; push; turnaround; fup; turnaround; root; pop]; % z position of primaries
end

%
% set up root crowns
%

N = round((365-firstCR(1))/delayRC(1)); % a guess how many root crowns are needed

delay0 = firstCR(1)+randn()*firstCR(2); 

for i = 1 : N % create N crown roots    
    
    crown{i} = [];
    NCR = round(nCR(1)+randn()*nCR(2));
    delay = delay0;
    alpha0 = rand()*2*pi; % start at some angle
    
    for j = 1 : NCR  % create a single root crown
        
        alpha = (2*pi/NCR)*j+alpha0;
        theta = parameters{4}.theta(1)+randn()*parameters{4}.theta(2);
        r = [letter('\',alpha); letter('+',theta)];
        
        if delay~=0
            dr = letter(301,[0 delay],shootborne); % delay
        else
            dr = taproot;
        end        
        crown{i} = [crown{i}; push; r; dr; pop];        

        delay = delay + delayCR(1)+randn()*delayCR(2);        
        
    end
    
    delay0 = delay0+delayRC(1)+randn()*delayRC(2); % next root crown
    
end

if N>0
    fup = letter('f',depth/2);
    crownroots = [ push; turnaround; fup; turnaround; crown{1}; pop;];
    cz = 0;
    for i = 2 : N
        cz = cz+dzRC(1)+randn()*dzRC(2);
        fup = letter('f',depth/2+cz);
        crownroots = [crownroots; push; turnaround; fup; turnaround; crown{i}; pop];
    end
else
    crownroots = [];
end

%
% set up root system
% 
rootsystem = [pitchdown;letter('f',depth);str; crownroots];  
rootsystem = updatePos(rootsystem);
